/*
 * Copyright (c) 2012-2014 Wind River Systems, Inc.
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdio.h>
#include <zephyr/kernel.h>
#include <zephyr/device.h>
#include <zephyr/devicetree.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/spi.h>
#include "l6470.h"

#define LED0_NODE DT_ALIAS(led0)
#define MOTOR_DRIVER DT_NODELABEL(motor_driver)

const struct device *const dev = DEVICE_DT_GET(MOTOR_DRIVER);

#define MY_GPIO0 DT_NODELABEL(gpio0)
#define GPIO_0_CS 11
const struct device *gpio0_dev = DEVICE_DT_GET(MY_GPIO0);

#define GPIO_11_NRESET_L6470	18

static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED0_NODE, gpios);

static struct spi_config spi_cfg = 
{
	.frequency = 2000000,
	.operation = SPI_WORD_SET(8),
	.slave = 0,
};

static void debug_print(const struct device *const dev, int count)
{
	uint16_t adc_out = l64x0_getparam_adc_out(dev);
	uint32_t speed = l64x0_getparam_speed(dev);
	uint32_t status = l64x0_get_status(dev);
	printk("%d: Speed: %u (0x%x), Status: 0x%x (%s%s%s%s%s%s%s%s%s%s%s%s%s), ADC_OUT %x\n",
	       count,
	       speed, speed, status,
	       (status & L6470_STATUS_SCK_MOD) ? "SCK_MOD " : "",
	       (status & L6470_STATUS_STEP_LOSS_B) ? "" : "STEP_LOSS_B ",
	       (status & L6470_STATUS_STEP_LOSS_A) ? "" : "STEP_LOSS_A ",
	       (status & L6470_STATUS_OCD) ? "" : "OCD ",
	       (status & L6470_STATUS_TH_SD) ? "" : "TH_SD ",
	       (status & L6470_STATUS_TH_WRN) ? "" : "TH_WRN ",
	       (status & L6470_STATUS_UVLO) ? "" : "UVLO ",
	       (status & L6470_STATUS_WRONG_CMD) ? "WRONG_CMD " : "",
	       (status & L6470_STATUS_NOTPERF_CMD) ? "NOTPERF_CMD " : "",
	       (status & L6470_STATUS_DIR) ? "DIR_f " : "DIR_r ",
	       (status & L6470_STATUS_SW_EVN) ? "SW_EVN " : "",
	       (status & L6470_STATUS_BUSY) ? "" : "BUSY ",
	       (status & L6470_STATUS_HiZ) ? "HiZ" : "",
	       adc_out);
}

int main(void)
{
	printf("Hello World! %s\n", CONFIG_BOARD);

	if (!device_is_ready(dev)) {
		printk("%s: device not ready.\n", dev->name);
		return;
	}

	const struct gpio_dt_spec n_stby = GPIO_DT_SPEC_GET(DT_NODELABEL(motor_driver), stby_gpios);
	if (!device_is_ready(n_stby.port)) {
		printk("%s: device not ready.\n", dev->name);
		return;
	}
	gpio_pin_configure_dt(&n_stby, GPIO_OUTPUT_ACTIVE);
	k_usleep(10);
	gpio_pin_set_dt(&n_stby, 0);
	
	l64x0_nop(dev);
	l64x0_nop(dev);
	l64x0_nop(dev);
	l64x0_nop(dev);
	l64x0_reset_device(dev);
	k_usleep(500);
	debug_print(dev, -1);

	/* K_VAL = (K_VAL_X + BEMF_COMP) * VSCOMP * K_THERM) * microstep */
	/* No compensation is active */
	/** The K_VALs */
	l64x0_setparam_kval_hold(dev, L64X0_KVAL_X_256TH(22));
	l64x0_setparam_kval_acc(dev, L64X0_KVAL_X_256TH(22));
	l64x0_setparam_kval_dec(dev, L64X0_KVAL_X_256TH(22));
	l64x0_setparam_kval_run(dev, L64X0_KVAL_X_256TH(22));
	

	while (1)
	{
		int status_response = l64x0_get_status(dev);
		printk("L6470 status 0x%0xX\n", status_response);
		
	}
	
}
